@@ -605,7 +605,8 @@ pg_data_type :
 parens_value_list : '(' VALUE(s /,/) ')'
     { $item[2] }
 
-parens_word_list : '(' WORD(s /,/) ')'
+
+parens_word_list : '(' name_with_opt_quotes(s /,/) ')'
     { $item[2] }
 
 field_size : '(' num_range ')' { $item{'num_range'} }
@@ -883,7 +884,7 @@ create_table : CREATE TABLE
 
 create_index : CREATE /index/i
 
-default_val  : DEFAULT /(\d+|'[^']*'|\w+\(.*?\))|\w+/
+default_val  : DEFAULT /(\d+|'[^']*'|\w+\(.*\))|\w+/
     { 
         my $val =  defined $item[2] ? $item[2] : '';
         $val    =~ s/^'|'$//g; 
