@@ -1,7 +1,7 @@
 package SQL::Translator::Parser::YAML;
 
 # -------------------------------------------------------------------
-# $Id: YAML.pm,v 1.6 2004/03/09 19:19:21 kycl4rk Exp $
+# $Id: YAML.pm,v 1.8 2006-03-10 14:55:19 grommit Exp $
 # -------------------------------------------------------------------
 # Copyright (C) 2002-4 SQLFairy Authors
 #
@@ -22,7 +22,7 @@ package SQL::Translator::Parser::YAML;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = sprintf "%d.%02d", q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/;
 
 use SQL::Translator::Schema;
 use SQL::Translator::Utils qw(header_comment);
@@ -34,7 +34,7 @@ sub parse {
     $data = Load($data);
     $data = $data->{'schema'};
 
-    warn Dumper( $data ) if $translator->debug;
+    warn "YAML data:",Dumper( $data ) if $translator->debug;
 
     my $schema = $translator->schema;
 
@@ -44,13 +44,16 @@ sub parse {
     my @tables = 
         map   { $data->{'tables'}{ $_->[1] } }
         sort  { $a->[0] <=> $b->[0] }
-        map   { [ $data->{'tables'}{ $_ }{'order'}, $_ ] }
+        map   { [ $data->{'tables'}{ $_ }{'order'} || 0, $_ ] }
         keys %{ $data->{'tables'} }
     ;
 
     for my $tdata ( @tables ) {
+  
         my $table = $schema->add_table(
-            name  => $tdata->{'name'},
+            map {
+              $tdata->{$_} ? ($_ => $tdata->{$_}) : ()
+            } (qw/name extra options/)
         ) or die $schema->error;
 
         my @fields = 
