@@ -1,7 +1,7 @@
 package SQL::Translator::Schema::Constraint;
 
 # ----------------------------------------------------------------------
-# $Id: Constraint.pm,v 1.13 2004/03/29 11:18:12 grommit Exp $
+# $Id: Constraint.pm,v 1.21 2007-03-01 22:15:59 duality72 Exp $
 # ----------------------------------------------------------------------
 # Copyright (C) 2002-4 SQLFairy Authors
 #
@@ -44,14 +44,14 @@ C<SQL::Translator::Schema::Constraint> is the constraint object.
 =cut
 
 use strict;
-use Class::Base;
 use SQL::Translator::Schema::Constants;
 use SQL::Translator::Utils 'parse_list_arg';
 
-use base 'Class::Base';
+use base 'SQL::Translator::Schema::Object';
+
 use vars qw($VERSION $TABLE_COUNT $VIEW_COUNT);
 
-$VERSION = sprintf "%d.%02d", q$Revision: 1.13 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.21 $ =~ /(\d+)\.(\d+)/;
 
 my %VALID_CONSTRAINT_TYPE = (
     PRIMARY_KEY, 1,
@@ -62,8 +62,16 @@ my %VALID_CONSTRAINT_TYPE = (
 );
 
 # ----------------------------------------------------------------------
-sub init {
 
+__PACKAGE__->_attributes( qw/
+    table name type fields reference_fields reference_table 
+    match_type on_delete on_update expression deferrable
+/);
+
+# Override to remove empty arrays from args.
+# t/14postgres-parser breaks without this.
+sub init {
+    
 =pod
 
 =head2 new
@@ -84,19 +92,9 @@ Object constructor.
 
 =cut
 
-    my ( $self, $config ) = @_;
-    my @fields = qw[ 
-        table name type fields reference_fields reference_table 
-        match_type on_delete on_update expression
-    ];
-
-    for my $arg ( @fields ) {
-        next unless $config->{ $arg };
-        next if ref $config->{ $arg } eq 'ARRAY' && ! @{ $config->{ $arg } };
-        defined $self->$arg( $config->{ $arg } ) or return;
-    }
-
-    return $self;
+    my $self = shift;
+    foreach ( values %{$_[0]} ) { $_ = undef if ref($_) eq "ARRAY" && ! @$_; }
+    $self->SUPER::init(@_);
 }
 
 # ----------------------------------------------------------------------
@@ -278,7 +276,7 @@ avoid the overload magic of the Field objects returned by the fields method.
 =cut
 
     my $self = shift;
-    return wantarray ? @{ $self->{'fields'} } : $self->{'fields'};
+    return wantarray ? @{ $self->{'fields'} || [] } : ($self->{'fields'} || '');
 }
 
 # ----------------------------------------------------------------------
@@ -524,6 +522,73 @@ Get or set the constraint's type.
 
     return $self->{'type'} || '';
 }
+
+# ----------------------------------------------------------------------
+sub equals {
+
+=pod
+
+=head2 equals
+
+Determines if this constraint is the same as another
+
+  my $isIdentical = $constraint1->equals( $constraint2 );
+
+=cut
+
+    my $self = shift;
+    my $other = shift;
+    my $case_insensitive = shift;
+    my $ignore_constraint_names = shift;
+    
+    return 0 unless $self->SUPER::equals($other);
+    return 0 unless $self->type eq $other->type;
+    unless ($ignore_constraint_names) {
+        return 0 unless $case_insensitive ? uc($self->name) eq uc($other->name) : $self->name eq $other->name;
+    }
+    return 0 unless $self->deferrable eq $other->deferrable;
+    #return 0 unless $self->is_valid eq $other->is_valid;
+    return 0 unless $case_insensitive ? uc($self->table->name) eq uc($other->table->name)
+    	: $self->table->name eq $other->table->name;
+    return 0 unless $self->expression eq $other->expression;
+    
+    # Check fields, regardless of order
+    my %otherFields = ();	# create a hash of the other fields
+    foreach my $otherField ($other->fields) {
+    	$otherField = uc($otherField) if $case_insensitive;
+    	$otherFields{$otherField} = 1;
+    }
+    foreach my $selfField ($self->fields) { # check for self fields in hash
+    	$selfField = uc($selfField) if $case_insensitive;
+    	return 0 unless $otherFields{$selfField};
+    	delete $otherFields{$selfField};
+    }
+    # Check all other fields were accounted for
+    return 0 unless keys %otherFields == 0;
+
+    # Check reference fields, regardless of order
+    my %otherRefFields = ();	# create a hash of the other reference fields
+    foreach my $otherRefField ($other->reference_fields) {
+    	$otherRefField = uc($otherRefField) if $case_insensitive;
+    	$otherRefFields{$otherRefField} = 1;
+    }
+    foreach my $selfRefField ($self->reference_fields) { # check for self reference fields in hash
+    	$selfRefField = uc($selfRefField) if $case_insensitive;
+    	return 0 unless $otherRefFields{$selfRefField};
+    	delete $otherRefFields{$selfRefField};
+    }
+    # Check all other reference fields were accounted for
+    return 0 unless keys %otherRefFields == 0;
+
+    return 0 unless $case_insensitive ? uc($self->reference_table) eq uc($other->reference_table) : $self->reference_table eq $other->reference_table;
+    return 0 unless $self->match_type eq $other->match_type;
+    return 0 unless $self->on_delete eq $other->on_delete;
+    return 0 unless $self->on_update eq $other->on_update;
+    return 0 unless $self->_compare_objects(scalar $self->options, scalar $other->options);
+    return 0 unless $self->_compare_objects(scalar $self->extra, scalar $other->extra);
+    return 1;
+}
+
 # ----------------------------------------------------------------------
 sub DESTROY {
     my $self = shift;
