@@ -1,7 +1,7 @@
 package SQL::Translator::Schema::View;
 
 # ----------------------------------------------------------------------
-# $Id: View.pm,v 1.9 2004/11/05 13:19:31 grommit Exp $
+# $Id: View.pm,v 1.14 2007-03-21 15:20:50 duality72 Exp $
 # ----------------------------------------------------------------------
 # Copyright (C) 2002-4 SQLFairy Authors
 #
@@ -50,7 +50,7 @@ use base 'SQL::Translator::Schema::Object';
 
 use vars qw($VERSION $TABLE_COUNT $VIEW_COUNT);
 
-$VERSION = sprintf "%d.%02d", q$Revision: 1.9 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.14 $ =~ /(\d+)\.(\d+)/;
 
 # ----------------------------------------------------------------------
 
@@ -103,7 +103,7 @@ names and keep them in order by the first occurrence of a field name.
         $self->{'fields'} = \@unique;
     }
 
-    return wantarray ? @{ $self->{'fields'} || [] } : $self->{'fields'};
+    return wantarray ? @{ $self->{'fields'} || [] } : ($self->{'fields'} || '');
 }
 
 # ----------------------------------------------------------------------
@@ -209,6 +209,47 @@ Get or set the view's schema object.
     return $self->{'schema'};
 }
 
+# ----------------------------------------------------------------------
+sub equals {
+
+=pod
+
+=head2 equals
+
+Determines if this view is the same as another
+
+  my $isIdentical = $view1->equals( $view2 );
+
+=cut
+
+    my $self = shift;
+    my $other = shift;
+    my $case_insensitive = shift;
+    my $ignore_sql = shift;
+    
+    return 0 unless $self->SUPER::equals($other);
+    return 0 unless $case_insensitive ? uc($self->name) eq uc($other->name) : $self->name eq $other->name;
+    #return 0 unless $self->is_valid eq $other->is_valid;
+    
+    unless ($ignore_sql) {
+        my $selfSql = $self->sql;
+        my $otherSql = $other->sql;
+        # Remove comments
+        $selfSql =~ s/--.*$//mg;
+        $otherSql =~ s/--.*$//mg;
+        # Collapse whitespace to space to avoid whitespace comparison issues
+        $selfSql =~ s/\s+/ /sg;
+        $otherSql =~ s/\s+/ /sg;
+        return 0 unless $selfSql eq $otherSql;
+    }
+    
+    my $selfFields = join(":", $self->fields);
+    my $otherFields = join(":", $other->fields);
+    return 0 unless $case_insensitive ? uc($selfFields) eq uc($otherFields) : $selfFields eq $otherFields;
+    return 0 unless $self->_compare_objects(scalar $self->extra, scalar $other->extra);
+    return 1;
+}
+
 # ----------------------------------------------------------------------
 sub DESTROY {
     my $self = shift;
